package com.arubanetworks.meridiansamples

import android.graphics.Color
import android.graphics.Matrix
import android.graphics.RectF
import android.os.Bundle
import android.view.*
import com.arubanetworks.meridian.location.MeridianLocation
import com.arubanetworks.meridian.location.MeridianOrientation
import com.arubanetworks.meridian.maps.*
import com.arubanetworks.meridian.requests.MapInfoRequest
import com.google.android.gms.maps.CameraUpdateFactory
import com.google.android.gms.maps.GoogleMap
import com.google.android.gms.maps.OnMapReadyCallback
import com.google.android.gms.maps.model.LatLng
import com.google.android.gms.maps.model.PolygonOptions
import kotlin.math.min
import kotlin.math.sqrt

/**
 * A Fragment to demonstrate the integration with Google maps.  NOTE: this sample requires a Google maps API key which needs
 * to be set in the manifest (android:value="ADD_YOUR_GOOGLE_MAPS_API_KEY_HERE").
 */

class GoogleMapsFragment : androidx.fragment.app.Fragment(), OnMapReadyCallback, MapView.MapEventListener {
    private lateinit var mGoogleMapView: com.google.android.gms.maps.MapView
    private lateinit var mMeridianMapView: MapView
    private lateinit var mGoogleMap: GoogleMap
    private lateinit var mMapInfo: MapInfo
    private var mGoogleMapHeading: Float = 0.toFloat()
    private var meridianMapLoaded = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        MapInfoRequest.Builder()
                .setMapKey(Application.MAP_KEY)
                .setListener { mapInfo ->
                    mMapInfo = mapInfo
                    addMapOverlay()
                }
                .setErrorListener {
                    // Handle error here
                }
                .build()
                .sendRequest()
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        val layout = inflater.inflate(R.layout.fragment_google_maps, container, false)
        mGoogleMapView = layout.findViewById(R.id.gm_googlemap)
        // Note that the Google map was preloaded in MainActivity because this can take 3+ seconds
        mGoogleMapView.onCreate(savedInstanceState)
        mGoogleMapView.getMapAsync(this)
        //setup meridian map
        mMeridianMapView = layout.findViewById(R.id.gm_meridian_map)
        mMeridianMapView.appKey = Application.APP_KEY
        mMeridianMapView.mapKey = Application.MAP_KEY
        mMeridianMapView.setMapEventListener(this)
        return layout
    }

    override fun onStart() {
        super.onStart()
        meridianMapLoaded = false

        if (mGoogleMapView != null) {
            mGoogleMapView.onStart()
        }
    }

    override fun onResume() {
        super.onResume()

        if (mGoogleMapView != null) {
            mGoogleMapView.onResume()
        }

        if (mMeridianMapView != null) {
            mMeridianMapView.onResume()
        }
    }

    override fun onPause() {
        super.onPause()

        if (mGoogleMapView != null) {
            mGoogleMapView.onPause()
        }

        if (mMeridianMapView != null) {
            mMeridianMapView.onPause()
        }
    }

    override fun onStop() {
        super.onStop()

        if (mGoogleMapView != null) {
            mGoogleMapView.onStop()
        }
    }

    override fun onLowMemory() {
        super.onLowMemory()

        if (mGoogleMapView != null) {
            mGoogleMapView.onLowMemory()
        }
    }

    override fun onSaveInstanceState(bundle: Bundle) {
        super.onSaveInstanceState(bundle)

        if (mGoogleMapView != null) {
            mGoogleMapView.onSaveInstanceState(bundle)
        }
    }

    override fun onDestroy() {
        super.onDestroy()

        if (mGoogleMapView != null) {
            mGoogleMapView.onDestroy()
        }

        if (mMeridianMapView != null) {
            mMeridianMapView.onDestroy()
            mMeridianMapView.setMapEventListener(null)
        }
    }

    override fun onMapReady(map: GoogleMap) {
        mGoogleMap = map
        try {
            map.isMyLocationEnabled = true
        } catch (ignore: SecurityException) {
        }

        addMapOverlay()
        mGoogleMapHeading = Math.toRadians(mGoogleMap.cameraPosition.bearing.toDouble()).toFloat()
        mGoogleMap.setOnCameraIdleListener {
            if (mGoogleMapView.visibility == View.VISIBLE) {
                val cameraPosition = mGoogleMap.cameraPosition
                if (cameraPosition.zoom > 18.0) {
                    mGoogleMapHeading = Math.toRadians(cameraPosition.bearing.toDouble()).toFloat()
                    if (meridianMapLoaded) {
                        resetMeridianMap()
                    }
                    mGoogleMapView.visibility = View.GONE
                    mMeridianMapView.visibility = View.VISIBLE
                    // zoom out a bit to smooth transition
                    mGoogleMap.animateCamera(CameraUpdateFactory.newLatLngZoom(LatLng(cameraPosition.target.latitude,
                            cameraPosition.target.longitude), 17.0f))
                }
            }
        }
    }

    private fun addMapOverlay() {
        if (::mMapInfo.isInitialized && this::mGoogleMap.isInitialized) {
            val topLeftPoint = mMapInfo.mapPointLatLong(0f, 0f)
            val topRightPoint = mMapInfo.mapPointLatLong(mMapInfo.width.toFloat(), 0f)
            val bottomLeftPoint = mMapInfo.mapPointLatLong(0f, mMapInfo.height.toFloat())
            val bottomRightPoint = mMapInfo.mapPointLatLong(mMapInfo.width.toFloat(), mMapInfo.height.toFloat())
            val topLeftLatLong = LatLng(topLeftPoint.x.toDouble(), topLeftPoint.y.toDouble())
            val topRightLatLong = LatLng(topRightPoint.x.toDouble(), topRightPoint.y.toDouble())
            val bottomLeftLatLong = LatLng(bottomLeftPoint.x.toDouble(), bottomLeftPoint.y.toDouble())
            val bottomRightLatLong = LatLng(bottomRightPoint.x.toDouble(), bottomRightPoint.y.toDouble())
            mGoogleMap.addPolygon(PolygonOptions()
                    .add(topLeftLatLong, topRightLatLong, bottomRightLatLong, bottomLeftLatLong)
                    .strokeColor(Color.RED)
                    .fillColor(Color.BLUE))
            mGoogleMap.moveCamera(CameraUpdateFactory.newLatLngZoom(topLeftLatLong, 16f))
        }
    }

    private fun resetMeridianMap() {
        // rotate map to match Google map orientation and zoom out
        val rotation = (2 * Math.PI - (mMeridianMapView.mapInfo!!.north + mGoogleMapHeading)).toFloat()
        mMeridianMapView.scrollToRect(RectF(0f, 0f, mMapInfo.width.toFloat(), mMapInfo.height.toFloat()), rotation, false)
    }
    //
    // MapViewListener methods
    //
    override fun onMapLoadStart() {meridianMapLoaded = false}
    override fun onMapLoadFinish() {
        mMapInfo = mMeridianMapView.mapInfo
        meridianMapLoaded = true
        resetMeridianMap()
    }
    override fun onPlacemarksLoadFinish() {}
    override fun onMapRenderFinish() {}
    override fun onMapLoadFail(tr: Throwable) {}
    override fun onMapTransformChange(transform: Matrix) {
        if (mMeridianMapView.visibility == View.VISIBLE && meridianMapLoaded) {
            // get current zoom scale value from transform
            val transformMatrix = FloatArray(9)
            transform.getValues(transformMatrix)
            // Compensate scale for rotation
            val mapScale = sqrt((transformMatrix[Matrix.MSCALE_X] * transformMatrix[Matrix.MSCALE_X] +
                    transformMatrix[Matrix.MSKEW_Y] * transformMatrix[Matrix.MSKEW_Y]).toDouble()).toFloat()

            // Minimum zoom scale required to fit the entire map on screen
            val minimumZoomScale = min(mMeridianMapView.width / mMapInfo.width,
                    mMeridianMapView.height / mMapInfo.height)
            // Dismiss the meridian map if we zoom out much farther than that.
            if (minimumZoomScale * 0.9 > mapScale) {
                mGoogleMapView.visibility = View.VISIBLE
                mMeridianMapView.visibility = View.GONE
                resetMeridianMap()
            }
        }
    }
    override fun onLocationUpdated(meridianLocation: MeridianLocation) {}
    override fun onOrientationUpdated(orientation: MeridianOrientation) {}
}
